@@ -3,25 +3,29 @@ use warnings;
 use Wx;
 
 package App::GUI::Cellgraph::Frame::Panel::Rules;
-use base qw/Wx::ScrolledWindow/;
+use base qw/Wx::Panel/;
 use App::GUI::Cellgraph::RuleGenerator;
-use App::GUI::Cellgraph::Widget::Rule;
+use App::GUI::Cellgraph::Widget::RuleInput;
 use App::GUI::Cellgraph::Widget::Action;
 use App::GUI::Cellgraph::Widget::ColorToggle;
+use Graphics::Toolkit::Color qw/color/;
 
 sub new {
     my ( $class, $parent, $state, $act_state ) = @_;
     my $self = $class->SUPER::new( $parent, -1);
-   
-    my $colors = [[255,255,255], [0,0,0]];
-    my $rule_cell_size = 20;
-    $self->{'rule_size'} = 3;
-    $self->{'rules'} = App::GUI::Cellgraph::RuleGenerator->new($self->{'rule_size'});
-    $self->{'call_back'} = sub {};
+
+    $self->{'rule_plate'} = Wx::ScrolledWindow->new( $self );
+    $self->{'rule_plate'}->ShowScrollbars(0,1);
+    $self->{'rule_plate'}->EnableScrolling(0,1);
+    $self->{'rule_plate'}->SetScrollRate( 1, 1 );
+    $self->{'call_back'}  = sub {};
+    $self->{'rule_square_size'} = 20;
+    $self->{'input_size'} = 3;
+    $self->{'state_count'} = 2;
+
 
     $self->{'rule_nr'}   = Wx::TextCtrl->new( $self, -1, 0, [-1,-1], [ 50, -1], &Wx::wxTE_PROCESS_ENTER );
-    $self->{'action_nr'} = Wx::TextCtrl->new( $self, -1, 22222222, [-1,-1], [ 85, -1], &Wx::wxTE_PROCESS_ENTER );
-    
+    $self->{'rule_nr'}->SetToolTip('number of currently displayed rule');
     $self->{'btn'}{'prev'}   = Wx::Button->new( $self, -1, '<',  [-1,-1], [30,25] );
     $self->{'btn'}{'next'}   = Wx::Button->new( $self, -1, '>',  [-1,-1], [30,25] );
     $self->{'btn'}{'sh_l'}   = Wx::Button->new( $self, -1, '<<', [-1,-1], [35,25] );
@@ -30,16 +34,12 @@ sub new {
     $self->{'btn'}{'inv'}    = Wx::Button->new( $self, -1, '!',  [-1,-1], [30,25] );
     $self->{'btn'}{'opp'}    = Wx::Button->new( $self, -1, '%',  [-1,-1], [30,25] );
     $self->{'btn'}{'rnd'}    = Wx::Button->new( $self, -1, '?',  [-1,-1], [30,25] );
-    $self->{'btn'}{'act_1'}  = Wx::Button->new( $self, -1, '1',  [-1,-1], [30,25] );
-    $self->{'btn'}{'act_2'}  = Wx::Button->new( $self, -1, '2',  [-1,-1], [30,25] );
-    $self->{'btn'}{'act_!'}  = Wx::Button->new( $self, -1, '!',  [-1,-1], [30,25] );
-    $self->{'btn'}{'act_?'}  = Wx::Button->new( $self, -1, '?',  [-1,-1], [30,25] );
 
-    $self->{'btn'}{'sym'}->SetToolTip('choose symmetric rule (every rule swaps result with symmetric partner)');
-    $self->{'btn'}{'inv'}->SetToolTip('choose inverted rule (every rule that produces white, goes black and vice versa)');
-    $self->{'btn'}{'opp'}->SetToolTip('choose opposite rule');
+    $self->{'btn'}{'sym'}->SetToolTip('choose symmetric rule (every partial rule swaps result with symmetric partner)');
+    $self->{'btn'}{'inv'}->SetToolTip('choose inverted rule (every partial rule that produces white, goes black and vice versa)');
+    $self->{'btn'}{'opp'}->SetToolTip('choose opposite rule ()');
     $self->{'btn'}{'rnd'}->SetToolTip('choose random rule');
-
+    
     my $std_attr = &Wx::wxALIGN_LEFT | &Wx::wxGROW | &Wx::wxALIGN_CENTER_HORIZONTAL;
     my $all_attr = &Wx::wxGROW | &Wx::wxALL | &Wx::wxALIGN_CENTER_HORIZONTAL;
 
@@ -63,62 +63,15 @@ sub new {
     $rf_sizer->AddSpacer(20);
     $rf_sizer->Add( 0, 1, &Wx::wxEXPAND | &Wx::wxGROW);
 
-    my $act_sizer = Wx::BoxSizer->new( &Wx::wxHORIZONTAL );
-    $act_sizer->AddSpacer( 12 );
-    $act_sizer->Add( Wx::StaticText->new( $self, -1, 'Active :' ), 0, $all_attr, 10 );        
-    $act_sizer->AddSpacer( 15 );
-    $act_sizer->Add( $self->{'btn'}{'act_!'}, 0, $all_attr, 5 );
-    $act_sizer->Add( $self->{'btn'}{'act_1'}, 0, $all_attr, 5 );
-    $act_sizer->Add( $self->{'btn'}{'act_2'}, 0, $all_attr, 5 );
-    $act_sizer->Add( $self->{'btn'}{'act_?'}, 0, $all_attr, 5 );
-    $act_sizer->AddSpacer( 15 );
-    $act_sizer->Add( $self->{'action_nr'},   0, $all_attr, 5 );
-    $act_sizer->Add( 0, 1, &Wx::wxEXPAND | &Wx::wxGROW);
-
     my $main_sizer = Wx::BoxSizer->new(&Wx::wxVERTICAL);
     $main_sizer->AddSpacer( 15 );
     $main_sizer->Add( $rule_sizer, 0, $std_attr, 20);
     $main_sizer->AddSpacer( 5 );
     $main_sizer->Add( $rf_sizer, 0, $std_attr, 20);
-    $main_sizer->AddSpacer( 15 );
-    $main_sizer->Add( $act_sizer, 0, $std_attr, 20);
-    $main_sizer->AddSpacer( 10 );
+    #$main_sizer->AddSpacer( 10 );
+    $main_sizer->Add( Wx::StaticLine->new( $self, -1), 0, $std_attr|&Wx::wxALL, 10 );
 
-    for my $rule_index (@{$self->{'rules'}{'input_nr'}}){
-        my $in_img = App::GUI::Cellgraph::Widget::Rule->new( $self, $rule_cell_size, 
-                                                             $self->{'rules'}{'in_list'}[$rule_index], [$colors->[1]] );
-        $in_img->SetToolTip('input pattern of partial rule Nr.'.($rule_index+1));
-                                                             
-        $self->{'result'}[$rule_index] = App::GUI::Cellgraph::Widget::ColorToggle->new( 
-                                                             $self, $rule_cell_size, $rule_cell_size, $colors, 0);
-                                                             
-        $self->{'result'}[$rule_index]->SetCallBack( sub { 
-                $self->{'rule_nr'}->SetValue( $self->get_rule_number ); $self->{'call_back'}->() 
-        });
-        $self->{'result'}[$rule_index]->SetToolTip('result of partial rule Nr.'.($rule_index+1));
-
-        $self->{'action'}[$rule_index] = App::GUI::Cellgraph::Widget::Action->new( $self, $rule_cell_size, [255, 255, 255] );
-        
-        $self->{'action'}[$rule_index]->SetCallBack( sub { 
-                $self->{'action_nr'}->SetValue( $self->get_action_number ); $self->{'call_back'}->() 
-        });
-        $self->{'action'}[$rule_index]->SetToolTip('transfer of activity by partial rule Nr.'.($rule_index+1));
-        
-        my $row_sizer = Wx::BoxSizer->new( &Wx::wxHORIZONTAL );
-        $row_sizer->AddSpacer(30);
-        $row_sizer->Add( $in_img, 0, &Wx::wxGROW);
-        $row_sizer->AddSpacer(15);
-        $row_sizer->Add( Wx::StaticText->new( $self, -1, ' => ' ), 0, &Wx::wxGROW | &Wx::wxLEFT );        
-        $row_sizer->AddSpacer(15);
-        $row_sizer->Add( $self->{'result'}[$rule_index], 0, &Wx::wxGROW | &Wx::wxLEFT );
-        $row_sizer->AddSpacer(40);
-        $row_sizer->Add( $self->{'action'}[$rule_index], 0, &Wx::wxGROW | &Wx::wxLEFT );
-        $row_sizer->Add( 0, 1, &Wx::wxEXPAND | &Wx::wxGROW);
-        $main_sizer->AddSpacer(15);
-        $main_sizer->Add( $row_sizer, 0, $std_attr, 10);
-    }
-    
-    $main_sizer->Add( 0, 1, &Wx::wxEXPAND | &Wx::wxGROW);
+    $main_sizer->Add( $self->{'rule_plate'}, 1, $std_attr, 0);
     $self->SetSizer( $main_sizer );
     
     Wx::Event::EVT_TEXT_ENTER( $self, $self->{'rule_nr'}, sub { $self->set_data( $self->{'rule_nr'}->GetValue ); $self->{'call_back'}->() });
@@ -132,65 +85,116 @@ sub new {
     Wx::Event::EVT_BUTTON( $self, $self->{'btn'}{'inv'},  sub { $self->invert_rule; $self->{'call_back'}->() } );
     Wx::Event::EVT_BUTTON( $self, $self->{'btn'}{'opp'},  sub { $self->opposite_rule; $self->{'call_back'}->() } );
     Wx::Event::EVT_BUTTON( $self, $self->{'btn'}{'rnd'},  sub { $self->random_rule; $self->{'call_back'}->() } );
-    Wx::Event::EVT_BUTTON( $self, $self->{'btn'}{'act_1'},sub { $self->init_action; $self->{'call_back'}->() } );
-    Wx::Event::EVT_BUTTON( $self, $self->{'btn'}{'act_2'},sub { $self->grid_action; $self->{'call_back'}->() } );
-    Wx::Event::EVT_BUTTON( $self, $self->{'btn'}{'act_?'},sub { $self->random_action; $self->{'call_back'}->() } );
-    Wx::Event::EVT_BUTTON( $self, $self->{'btn'}{'act_!'},sub { $self->invert_action; $self->{'call_back'}->() } );
 
     Wx::Event::EVT_TEXT_ENTER( $self, $self->{'rule_nr'}, sub {
         my ($self, $cmd) = @_;
         my $new_value = $cmd->GetString;
-        my $old_value = $self->{'rules'}->nr_from_list( $self->get_list );
-        $self->set_rule( $new_value ) if $new_value != $old_value;
+        my $old_value = $self->{'rules'}->nr_from_output_list( $self->get_output_list );
+        return if $new_value == $old_value;
+        $self->set_rule( $new_value );
+        $self->{'call_back'}->();
         
     });
-    Wx::Event::EVT_TEXT_ENTER( $self, $self->{'action_nr'}, sub {
-        my ($self, $cmd) = @_;
-        my $new_value = $cmd->GetString;
-        my $old_value = $self->nr_from_action_list( $self->get_action_list );
-        $self->set_action( $new_value ) if $new_value != $old_value;
-    });
-
-    $self->init();
+    $self->regenerate_rules;
+    $self->init;
     $self;
 }
 
+sub regenerate_rules {
+    my ($self, $data) = @_;
+    return if ref $data eq 'HASH' and $self->{'state_count'} == $data->{'global'}{'state_count'}
+                                  and $self->{'input_size'} == $data->{'global'}{'input_size'};
+    $self->{'state_count'} = $data->{'global'}{'state_count'} if ref $data eq 'HASH';
+    $self->{'input_size'} = $data->{'global'}{'input_size'} if ref $data eq 'HASH';
+    $self->{'rules'} = App::GUI::Cellgraph::RuleGenerator->new( $self->{'input_size'}, $self->{'state_count'} );
+    $self->{'state_colors'} = [map {[$_->rgb]} color('white')->gradient_to('black', $self->{'state_count'})];
+    my @input_colors = map {[map { $self->{'state_colors'}[$_] } @$_ ]} @{$self->{'rules'}{'input_list'}};
+
+    my $refresh = 0;
+    if (exists $self->{'rule_img'}){
+        $self->{'plate_sizer'}->Clear(1);
+        $self->{'rule_img'} = [];
+        $self->{'arrow'} = [];
+        $self->{'rule_result'} = [];
+        # map { $_->Destroy} @{$self->{'rule_img'}}, @{$self->{'rule_result'}}, @{$self->{'arrow'}};
+        $refresh = 1;
+    } else {
+        $self->{'plate_sizer'} = Wx::BoxSizer->new(&Wx::wxVERTICAL);
+        $self->{'rule_plate'}->SetSizer( $self->{'plate_sizer'} );
+    }
+    my $std_attr = &Wx::wxALIGN_LEFT | &Wx::wxGROW | &Wx::wxALIGN_CENTER_HORIZONTAL;
+    for my $rule_index ($self->{'rules'}->part_rule_iterator){
+        $self->{'rule_img'}[$rule_index] = App::GUI::Cellgraph::Widget::RuleInput->new( 
+                                           $self->{'rule_plate'}, $self->{'rule_square_size'}, $input_colors[$rule_index] );
+        $self->{'rule_img'}[$rule_index]->SetToolTip('input pattern of partial rule Nr.'.($rule_index+1));
+        
+        $self->{'rule_result'}[$rule_index] = App::GUI::Cellgraph::Widget::ColorToggle->new( 
+                                         $self->{'rule_plate'}, $self->{'rule_square_size'}, $self->{'rule_square_size'}, 
+                                         $self->{'state_colors'}, 0 );
+        $self->{'rule_result'}[$rule_index]->SetCallBack( sub { 
+                $self->{'rule_nr'}->SetValue( $self->get_rule_number ); $self->{'call_back'}->() 
+        });
+        $self->{'rule_result'}[$rule_index]->SetToolTip('result of partial rule '.($rule_index+1));
+
+        $self->{'arrow'}[$rule_index] = Wx::StaticText->new( $self->{'rule_plate'}, -1, ' => ' );
+    }
+    for my $rule_index ($self->{'rules'}->part_rule_iterator){
+        my $row_sizer = Wx::BoxSizer->new( &Wx::wxHORIZONTAL );
+        $row_sizer->AddSpacer(30);
+        $row_sizer->Add( $self->{'rule_img'}[$rule_index], 0, &Wx::wxGROW);
+        $row_sizer->AddSpacer(15);
+        $row_sizer->Add( $self->{'arrow'}[$rule_index], 0, &Wx::wxGROW | &Wx::wxLEFT );        
+        $row_sizer->AddSpacer(15);
+        $row_sizer->Add( $self->{'rule_result'}[$rule_index], 0, &Wx::wxGROW | &Wx::wxLEFT );
+        $row_sizer->AddSpacer(40);
+        $row_sizer->Add( 0, 1, &Wx::wxEXPAND | &Wx::wxGROW);
+        $self->{'plate_sizer'}->AddSpacer(15);
+        $self->{'plate_sizer'}->Add( $row_sizer, 0, $std_attr, 10); # ->Insert(4,
+    }
+    $self->Layout if $refresh;
+}
+
 sub SetCallBack {
     my ($self, $code) = @_;
     return unless ref $code eq 'CODE';
     $self->{'call_back'} = $code;
 }
 
-sub get_list {
+sub get_output_list {
     my ($self) = @_;
-    map { $self->{'result'}[$_]->GetValue } @{$self->{'rules'}{'input_nr'}};
+    map { $self->{'rule_result'}[$_]->GetValue } $self->{'rules'}->part_rule_iterator;
 }
-sub get_rule_number { $_[0]->{'rules'}->nr_from_list( $_[0]->get_list ) }
+sub get_map {
+    my ($self) = @_;
+    my %map = map { $self->{'rules'}->input_pattern_from_nr($_) => 
+                    $self->{'rule_result'}[$_]->GetValue } $self->{'rules'}->part_rule_iterator;
+    \%map;
+}
+sub get_rule_number { $_[0]->{'rules'}->nr_from_output_list( $_[0]->get_output_list ) }
 
 sub set_rule {
     my ($self) = shift;
     my ($rule, @list);
     if (@_ == 1) {
         $rule = shift;
-        @list = $self->{'rules'}->list_from_nr( $rule );
+        @list = $self->{'rules'}->output_list_from_nr( $rule );
     } else {
         @list = @_;
-        $rule = $self->{'rules'}->nr_from_list( @list );
+        $rule = $self->{'rules'}->nr_from_output_list( @list );
     }
-    $self->{'result'}[$_]->SetValue( $list[$_] ) for 0 .. $#list;
+    $self->{'rule_result'}[$_]->SetValue( $list[$_] ) for 0 .. $#list;
     $self->{'rule_nr'}->SetValue( $rule );
 }
 
-sub init { $_[0]->set_data( { nr => 18, size => 3, action => 22222222 } ) }
+sub init { $_[0]->set_data( { nr => 18, size => 3, avg => 0 } ) }
 
 sub get_data {
     my ($self) = @_;
     {
-        f => [$self->get_list],
+        f => [$self->get_output_list],
         nr => $self->{'rule_nr'}->GetValue,
-        size => 3,
-        action => $self->{'action_nr'}->GetValue,
-        action_f => [$self->get_action_list],
+        size => $self->{'input_size'},
+        avg => $self->{'rules'}{'avg'},
     }
 }    
 
@@ -198,7 +202,6 @@ sub set_data {
     my ($self, $data) = @_;
     return unless ref $data eq 'HASH' and exists $data->{'nr'};
     $self->set_rule( $data->{'nr'} );
-    $self->set_action( $data->{'action'} );
 }    
 
 sub prev_rule      { $_[0]->set_rule( $_[0]->{'rules'}->prev_nr( $_[0]->{'rule_nr'}->GetValue ) ) }
@@ -212,51 +215,4 @@ sub symmetric_rule { $_[0]->set_rule( $_[0]->{'rules'}->symmetric_nr( $_[0]->{'r
 sub invert_rule    { $_[0]->set_rule( $_[0]->{'rules'}->inverted_nr( $_[0]->{'rule_nr'}->GetValue ) ) }
 sub random_rule    { $_[0]->set_rule( $_[0]->{'rules'}->random_nr ) }
 
-sub get_action_number { join '', reverse $_[0]->get_action_list }
-sub get_action_list {
-    my ($self) = @_;
-    map { $self->{'action'}[$_]->GetValue } @{$self->{'rules'}{'input_nr'}};
-}
-
-sub set_action {
-    my ($self) = shift;
-    my ($nr, @list);
-    if (@_ == 1) {
-        $nr = shift;
-        @list = $self->list_from_action_nr( $nr );
-    } else {
-        @list = @_;
-        $nr = $self->nr_from_action_list( @list );
-    }
-    $self->{'action_nr'}->SetValue( $nr );
-    $self->{'action'}[$_]->SetValue( $list[$_] ) for 0 .. $#list;
-}
-
-sub init_action {
-    my ($self) = @_;
-    my @list = map { $self->{'action'}[$_]->init } @{$self->{'rules'}{'input_nr'}};
-    $self->{'action_nr'}->SetValue( $self->nr_from_action_list( @list ) );
-}
-
-sub grid_action {
-    my ($self) = @_;
-    my @list = map { $self->{'action'}[$_]->grid } @{$self->{'rules'}{'input_nr'}};
-    $self->{'action_nr'}->SetValue( $self->nr_from_action_list( @list ) );
-}
-
-sub random_action {
-    my ($self) = @_;
-    my @list =  map { $self->{'action'}[$_]->rand } @{$self->{'rules'}{'input_nr'}};
-    $self->{'action_nr'}->SetValue( $self->nr_from_action_list( @list ) );
-}
-
-sub invert_action {
-    my ($self) = @_;
-    my @list = map { $self->{'action'}[$_]->invert } @{$self->{'rules'}{'input_nr'}};
-    $self->{'action_nr'}->SetValue( $self->nr_from_action_list( @list ) );
-}
-
-sub list_from_action_nr { reverse split '', $_[1]}
-sub nr_from_action_list { shift @_; join '', reverse @_ }
-
 1;
