@@ -4,20 +4,21 @@ use Wx;
 
 package App::GUI::Cellgraph::Frame::Panel::Start;
 use base qw/Wx::Panel/;
-use App::GUI::Cellgraph::Widget::Rule;
 use App::GUI::Cellgraph::Widget::ColorToggle;
 use App::GUI::Cellgraph::Widget::SliderCombo;
+use Graphics::Toolkit::Color qw/color/;
 
 sub new {
-    my ( $class, $parent, $state, $act_state ) = @_;
-    # my $x = 10;
-    # my $y = 10;
+    my ( $class, $parent ) = @_;
     my $self = $class->SUPER::new( $parent, -1);
     
-    my $colors = [[255,255,255], [0,0,0]];
+    
+    $self->{'state_count'} = 2;
+    $self->{'state_colors'} = [map {[$_->rgb]} color('white')->gradient_to('black', $self->{'state_count'})];
+
     my $rule_cell_size = 20;
     $self->{'length'} = my $length = 20;
-    $self->{'switch'}   = [ map { App::GUI::Cellgraph::Widget::ColorToggle->new( $self, $rule_cell_size, $rule_cell_size, $colors, 0) } 1 .. $length];
+    $self->{'switch'}   = [ map { App::GUI::Cellgraph::Widget::ColorToggle->new( $self, $rule_cell_size, $rule_cell_size, $self->{'state_colors'}, 0) } 1 .. $length];
     $self->{'start_int'}  = Wx::TextCtrl->new( $self, -1, 1, [-1,-1], [ 78, -1] );
     $self->{'start_int'}->SetToolTip('condensed content of start row');
     $self->{'repeat_start'} = Wx::CheckBox->new( $self, -1, '  Repeat');
@@ -25,14 +26,10 @@ sub new {
     $self->{'btn'}{'next'}  = Wx::Button->new( $self, -1, '>',  [-1,-1], [30,25] );
     $self->{'btn'}{'one'}   = Wx::Button->new( $self, -1, '1',  [-1,-1], [30,25] );
     $self->{'btn'}{'rnd'}   = Wx::Button->new( $self, -1, '?',  [-1,-1], [30,25] );
-    $self->{'grid_lbl'} = Wx::StaticText->new( $self, -1, 'Grid :');
     #$self->{'rule_size_lbl'} = Wx::StaticText->new( $self, -1, 'Size :');
     #$self->{'rule_type_lbl'} = Wx::StaticText->new( $self, -1, 'Rules :');
-    $self->{'cell_size_lbl'} = Wx::StaticText->new( $self, -1, 'Size :');
-    $self->{'grid'}      = Wx::ComboBox->new( $self, -1, 'lines', [-1,-1],[95, -1], ['lines', 'gaps', 'no']);
     #$self->{'rule_size'} = Wx::ComboBox->new( $self, -1, 3,        [-1,-1],[65, -1], [2, 3, 4, 5], &Wx::wxTE_READONLY);
     #$self->{'rule_type'} = Wx::ComboBox->new( $self, -1, 'pattern', [-1,-1],[110, -1], [qw/pattern average median/], &Wx::wxTE_READONLY);
-    $self->{'cell_size'} = Wx::ComboBox->new( $self, -1, '3', [-1,-1],[75, -1], [qw/1 2 3 4 5 6 7 8 9 10 12 14 16 18 20 25 30/], &Wx::wxTE_READONLY);
     $self->{'call_back'} = sub {};
     
     #$self->{'rule_type'}->SetToolTip('set rule type');
@@ -41,7 +38,6 @@ sub new {
     Wx::Event::EVT_BUTTON( $self, $self->{'btn'}{'next'}, sub { $self->next_start;  $self->{'call_back'}->() }) ;
     Wx::Event::EVT_BUTTON( $self, $self->{'btn'}{'one'},  sub { $self->reset_start; $self->{'call_back'}->() }) ;
     Wx::Event::EVT_BUTTON( $self, $self->{'btn'}{'rnd'},  sub { $self->random_start;$self->{'call_back'}->() }) ;
-    Wx::Event::EVT_COMBOBOX( $self, $self->{$_}, sub { $self->{'call_back'}->() }) for qw/grid cell_size /;# rule_size rule_type
     Wx::Event::EVT_CHECKBOX( $self, $self->{$_}, sub { $self->{'call_back'}->() }) for qw/repeat_start/;
     $_->SetCallBack( sub { $self->{'start_int'}->SetValue( $self->get_number ); $self->{'call_back'}->() }) for @{$self->{'switch'}};
     
@@ -49,26 +45,6 @@ sub new {
     my $row_attr = $std_attr | &Wx::wxLEFT;
     my $all_attr = $std_attr | &Wx::wxALL;
 
-    my $grid_sizer = Wx::BoxSizer->new( &Wx::wxHORIZONTAL );
-    $grid_sizer->AddSpacer( 23 );
-    $grid_sizer->Add( $self->{'grid_lbl'}, 0, $all_attr, 7);
-    $grid_sizer->Add( $self->{'grid'}, 0, $row_attr, 8);
-    $grid_sizer->AddSpacer( 31 );
-    $grid_sizer->Add( $self->{'cell_size_lbl'}, 0, $all_attr, 7);
-    $grid_sizer->AddSpacer( 3 );
-    $grid_sizer->Add( $self->{'cell_size'}, 0, $row_attr, 8);
-    $grid_sizer->Add( 0, 1, &Wx::wxEXPAND | &Wx::wxGROW);
-
-    #~ my $rule_sizer = Wx::BoxSizer->new( &Wx::wxHORIZONTAL );
-    #~ $rule_sizer->AddSpacer( 12 );
-    #~ $rule_sizer->Add( $self->{'rule_type_lbl'}, 0, $all_attr, 10);
-    #~ $rule_sizer->Add( $self->{'rule_type'}, 0, $all_attr, 5);
-    #~ $rule_sizer->AddSpacer( 8 );
-    #~ $rule_sizer->Add( $self->{'rule_size_lbl'}, 0, $all_attr, 10);
-    #~ $rule_sizer->AddSpacer( 13 );
-    #~ $rule_sizer->Add( $self->{'rule_size'}, 0, $all_attr, 5);
-    #~ $rule_sizer->Add( 0, 1, &Wx::wxEXPAND | &Wx::wxGROW);
-
     my $int_sizer = Wx::BoxSizer->new( &Wx::wxHORIZONTAL );
     $int_sizer->AddSpacer( 7 );
     $int_sizer->Add( Wx::StaticText->new( $self, -1, 'First Row: ' ), 0, &Wx::wxGROW | &Wx::wxALL, 10 );        
@@ -85,19 +61,12 @@ sub new {
     $io_sizer->Add( 0, 1, &Wx::wxEXPAND | &Wx::wxGROW);
     
     my $main_sizer = Wx::BoxSizer->new(&Wx::wxVERTICAL);
-    $main_sizer->AddSpacer( 20 );
-    $main_sizer->Add( $grid_sizer, 0, $std_attr, 0);
-    $main_sizer->AddSpacer( 25 );
-    #~ $main_sizer->Add( Wx::StaticLine->new( $self, -1), 0, $row_attr|&Wx::wxRIGHT, 20 );
-    #~ $main_sizer->AddSpacer( 25 );
-    #~ $main_sizer->Add( $rule_sizer, 0, $std_attr, 0);
-    $main_sizer->AddSpacer( 25 );
-    $main_sizer->Add( Wx::StaticLine->new( $self, -1), 0, $row_attr|&Wx::wxRIGHT, 20 );
-    $main_sizer->AddSpacer( 25 );
+    $main_sizer->AddSpacer( 15 );
     $main_sizer->Add( $int_sizer, 0, $std_attr, 20);
     $main_sizer->AddSpacer(20);
     $main_sizer->Add( $io_sizer, 0, $std_attr, 0);
     $main_sizer->Add( $self->{'repeat_start'}, 0, $all_attr, 23);
+    $main_sizer->Add( Wx::StaticLine->new( $self, -1), 0, $row_attr|&Wx::wxRIGHT, 20 );
 
  #   $main_sizer->Add( $self->{'rule_size'}, 0, $row_attr, 23);
     $main_sizer->Add( 0, 1, &Wx::wxEXPAND | &Wx::wxGROW);
@@ -106,6 +75,15 @@ sub new {
     $self;
 }
 
+sub regenerate_cells {
+    my ($self, $data) = @_;
+    return if ref $data eq 'HASH' and $self->{'state_count'} == $data->{'global'}{'state_count'};
+    $self->{'state_count'} = $data->{'global'}{'state_count'};
+    $self->{'state_colors'} = [map {[$_->rgb]} color('white')->gradient_to('black', $self->{'state_count'})];
+    $self->{'switch'}[$_]->SetColors( $self->{'state_colors'} ) for 0 .. $self->{'length'} - 1;
+}    
+
+
 sub get_number {
     my ($self) = @_;
     my $number = 0;
@@ -120,13 +98,13 @@ sub get_list {
     my ($self) = @_;
     my @list = map { $self->{'switch'}[$_]->GetValue } 0 .. $self->{'length'} - 1;
     pop @list while @list and not $list[-1];
-    if ($self->{'repeat_start'}->GetValue){ unshift @list, [] }
-    else                                  { shift @list while @list and not $list[0] }
+    # remove starting 0
+    unless ($self->{'repeat_start'}->GetValue){ shift @list while @list and not $list[0] }
     @list;
 }
 
 
-sub init        { $_[0]->set_data({ value => 1, grid => 'lines', cell_size => 3 }) }
+sub init        { $_[0]->set_data({ value => 1 }) }
 sub reset_start { $_[0]->set_start_row(1) }
 
 sub get_data {
@@ -134,8 +112,7 @@ sub get_data {
     {
         list => [$self->get_list],
         value => $self->{'start_int'}->GetValue,
-        cell_size => $self->{'cell_size'}->GetValue,
-        grid => $self->{'grid'}->GetValue,
+        repeat => $self->{'repeat_start'}->GetValue,
     }
 }
 
@@ -143,8 +120,6 @@ sub set_data {
     my ($self, $data) = @_;
     return unless ref $data eq 'HASH';
     $self->set_start_row( $data->{'value'} );
-    $self->{'grid'}->SetValue( $data->{'grid'} );
-    $self->{'cell_size'}->SetValue( $data->{'cell_size'} );
 }
 
 sub set_start_row {
@@ -182,14 +157,14 @@ sub prev_start {
     my ($self) = @_;
     my $int = $self->{'start_int'}->GetValue;
     $int-- if $int > 1;
-    $self->set_data( $int );
+    $self->set_start_row( $int );
 }
 
 sub next_start {
     my ($self) = @_;
     my $int = $self->{'start_int'}->GetValue;
     $int++;
-    $self->set_data( $int );
+    $self->set_start_row( $int );
 }
 
 1;
