@@ -26,6 +26,9 @@ sub load {
 
 sub write {
     my ($file, $data) = @_;
+    return unless ref $data eq 'HASH';
+    delete $data->{'rules'}{'f'};
+    delete $data->{'start'}{'list'};
     open my $FH, '>', $file or return "could not write $file: $!";
     for my $main_key (sort keys %$data){
         say $FH "\n  [$main_key]\n";
